package com.quadbase.article;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class BenchmarkRunner {
	private List<Integer> paramList, paramListSynchronized;

	/**
	 * launches the multi-threaded benchmark
	 * 
	 * @param args default argument, no arguments needed for this app
	 */
	public static void main(String[] args) {
		new BenchmarkRunner();
	}

	public BenchmarkRunner() {
		// set the number of application threads that will be used for export
		// set as many threads as your CPU can handle
		// using more threads than your CPU has is counter-productive
		// for instance for AMD 5950X, we'll use 32 threads
		int threadCount = 32;

		// fetch the parameter list
		// a reference of this list will be passed to all threads
		paramList = new ArrayList<>();
		for (int k = 1; k <= 50; k++) {
			paramList.add(k);
		}

		paramListSynchronized = Collections.synchronizedList(paramList);

		// we're starting the export, save the current system time in milliseconds
		long starttime = System.currentTimeMillis();

		// create a new Fixed Thread Pool in the size of the threadCount
		ExecutorService executor = Executors.newFixedThreadPool(threadCount);
		List<Future<?>> futures = new ArrayList<>();

		// start all threads one by one
		for (int i = 1; i <= threadCount; i++) {
			// create a new thread
			// initialize the ChartBenchmarkTemplate class using it's only constructor
			// pass the paramList as a reference
			Future<?> f = executor
					.submit(new ChartExporter(i, "templates/StatementChart.pac", paramListSynchronized));

			// add to the futures list so we have a list of all threads we launched
			futures.add(f);
		}

		// iterate over the threads
		for (Future<?> f : futures) {
			try {
				// wait for the thread to finish
				f.get();
			} catch (InterruptedException e) {
				// basic error handling, use something better in your project
				e.printStackTrace();
			} catch (ExecutionException e) {
				// basic error handling, use something better in your project
				e.printStackTrace();
			}
		}

		// all threads have finished, print out the time difference (in milliseconds)
		// and exit the application
		System.out.println("--- All threads completed in " + (System.currentTimeMillis() - starttime) + " miliseconds");
		// we're done, exit the application
		System.exit(0);
	}
}
