package com.quadbase.article;

import java.applet.Applet;
import java.util.List;

import quadbase.ChartAPI.QbChart;

public class ChartExporter implements Runnable {
	private String template = null;
	private List<Integer> params;
	private int threadid;

	/**
	 * initialize the chart export thread
	 * 
	 * @param threadid     the number of the thread. Informative. Only used in a
	 *                     println
	 * @param templateName the name of the chart template that will be exported
	 * @param params       the list of all remaining parameters waiting to be
	 *                     exported
	 */
	public ChartExporter(int threadid, String templateName, List<Integer> params) {
		// do not use EspressManager for the export
		QbChart.setEspressManagerUsed(false);
		this.threadid = threadid;
		this.template = templateName;
		this.params = params;
	}

	/**
	 * this method is run when the thread is started
	 */
	public void run() {
		export();
	}

	/**
	 * Get a parameter out of the params list and export it
	 */
	public void export() {
		try {
			// prepare a variable that will store the parameter
			Object queryParams[] = new Object[1];

			// while the params list still contains parameters to be exported, take one
			// parameter and export it
			while (params.size() > 0) {
				// it's important to handle the params list in a synchronized block to prevent
				// concurrent modification
				// synchronize by the params list only, all other commands can run in parallel
				synchronized (params) {
					// take and remove the last parameter from the params list and save it in this
					// thread
					queryParams[0] = params.remove(params.size() - 1);
				}

				// just prints an informative message
				System.out.println("Thread nr. " + threadid + " exporting chart ID " + queryParams[0]);

				// load the chart from the template, apply the parameter
				QbChart chart = new QbChart((Applet) null, template, queryParams);

				// export the chart
				chart.export(QbChart.PNG, "dist/StatementChartID_" + queryParams[0] + ".png");
			}
		} catch (Exception e) {
			// basic error handling, use something better in your project
			e.printStackTrace();
		}
	}

}
